--- FL/names.h.original	2008-09-10 20:56:49.000000000 -0300
+++ FL/names.h	2008-09-22 11:11:14.000000000 -0300
@@ -56,6 +56,7 @@
   "FL_DND_DRAG",
   "FL_DND_LEAVE",
   "FL_DND_RELEASE",
+  "FL_TOOLTIP"
 };
 
 char *fl_fontnames[] =
--- FL/Enumerations.H.original	2008-09-18 16:09:34.000000000 -0300
+++ FL/Enumerations.H	2008-09-22 11:10:42.000000000 -0300
@@ -294,7 +294,17 @@
    *  If the widget returns 1, it will receive the data in the immediately 
    *  following FL_PASTE event.
    */
-  FL_DND_RELEASE	= 23
+  FL_DND_RELEASE	= 23,
+
+
+  /** The widget under the cursor has a chance to display a tooltip. 
+   *  When a widget receives this event, it can modify its tooltip. The
+   *  modified tooltip will be drawn. If the widget removes its tooltip,
+   *  the parent with a tooltip will be drawn.
+   *
+   *  \sa Fl_Widget::tooltip(const char *t)
+   */
+  FL_TOOLTIP        = 24
 };
 
 /** \name When Conditions */
--- src/Fl_Group.cxx.original	2008-09-17 06:18:52.000000000 -0300
+++ src/Fl_Group.cxx	2008-09-22 11:10:42.000000000 -0300
@@ -278,6 +278,10 @@
     }
     return 1;
 
+  case FL_TOOLTIP:
+    if (tooltip()) return 1;
+    return 0;
+
   default:
     // For all other events, try to give to each child, starting at focus:
     for (i = 0; i < children(); i ++)
--- src/Fl_Tooltip.cxx.original	2008-09-16 04:26:22.000000000 -0300
+++ src/Fl_Tooltip.cxx	2008-09-22 11:10:42.000000000 -0300
@@ -159,6 +159,11 @@
   for (;;) {
     if (!tw) {exit_(0); return;}
     if (tw == widget_) return;
+
+    // Send the FL_TOOLTIP event to tw. The return value technically does not matter.
+	// The action to take will depend on the state of its tooltip.
+	tw->handle(FL_TOOLTIP);
+
     if (tw->tooltip()) break;
     tw = tw->parent();
   }
